@@ -72,7 +72,7 @@ solve(const char *equation)
 }
 
 static void
-usage(int show_gtk)
+usage(gboolean show_application, gboolean show_gtk)
 {
     fprintf(stderr,
             /* Description on how to use gcalctool displayed on command-line */
@@ -106,15 +106,18 @@ usage(int show_gtk)
                 "\n\n");
     }
 
-    fprintf(stderr,
-            /* Description on gcalctool application options displayed on command-line */
-            _("Application Options:\n"
-              "  -u, --unittest                  Perform unit tests\n"
-              "  -s, --solve <equation>          Solve the given equation"));
-    fprintf(stderr,
-            "\n\n");
+    if (show_application) {
+        fprintf(stderr,
+                /* Description on gcalctool application options displayed on command-line */
+                _("Application Options:\n"
+                  "  -u, --unittest                  Perform unit tests\n"
+                  "  -s, --solve <equation>          Solve the given equation"));
+        fprintf(stderr,
+                "\n\n");
+    }
 }
 
+
 void
 get_options(int argc, char *argv[])
 {
@@ -132,11 +135,15 @@ get_options(int argc, char *argv[])
         else if (strcmp(arg, "-h") == 0 ||
                  strcmp(arg, "-?") == 0 ||
                  strcmp(arg, "--help") == 0) {
-            usage(FALSE);
+            usage(TRUE, FALSE);
             exit(0);
         }
         else if (strcmp(arg, "--help-all") == 0) {
-            usage(TRUE);
+            usage(TRUE, TRUE);
+            exit(0);
+        }
+        else if (strcmp(arg, "--help-gtk") == 0) {
+            usage(FALSE, TRUE);
             exit(0);
         }
         else if (strcmp(arg, "-s") == 0 ||
@@ -161,7 +168,7 @@ get_options(int argc, char *argv[])
                     /* Error printed to stderr when user provides an unknown command-line argument */
                     _("Unknown argument '%s'"), arg);
             fprintf(stderr, "\n");
-            usage(FALSE);
+            usage(TRUE, FALSE);
             exit(1);
         }
     }
